/*
 * CommandDriverV.cpp
 *		- "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 *      99/05/24 KISAKA Toshihiro       modify for mwcc compatiblility
 *
 */

#include "stdafx.h"
#include "CommandDriver.h"

#include "ScenarioDecoder.h"
#include "ScenarioParser.h"
#include "ExpressionDriver.h"
#include "GraphicDriver.h"
#include "UnitmapDriver.h"

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
using namespace ALSD;
using namespace ALSD_CMDDRV;
#endif
/**********************************************************************
	V command
**********************************************************************/
void CCommandDriver::_CommandV(void)
{
	switch( m_lpSD->GetParser()->GetC() ){
		case 'A':
			CommandVA(); break;
		case 'B':
			CommandVB(); break;
		case 'C':
			CommandVC(); break;
		case 'E':
			CommandVE(); break;
		case 'F':
			CommandVF(); break;
		case 'G':
			CommandVG(); break;
		case 'H':
			CommandVH(); break;
		case 'I':
			switch( m_lpSD->GetParser()->GetC() ){
				case 'C':
					CommandVIC(); break;
				case 'P':
					CommandVIP(); break;
				default:
					throw EALSD_Msg( "error: VI? unknown command\n" );
			}
			break;
		case 'P':
			CommandVP(); break;
		case 'R':
			CommandVR(); break;
		case 'S':
			CommandVS(); break;
		case 'T':
			CommandVT(); break;
		case 'V':
			CommandVV(); break;
		case 'W':
			CommandVW(); break;
		case 'X':
			CommandVX(); break;
		case 'Z':
			CommandVZ(); break;
		default:
			throw EALSD_Msg( "error: V? unknown command\n" );
	}
}
/**********************************************************************
	VA
**********************************************************************/
void CCommandDriver::CommandVA( void )
{
	long no = m_lpSD->GetParser()->GetC();
	long p1 = m_lpED->GetExpValue();
	long p2 = 0, p3 = 0;
	long *var1 = NULL, *var2 = NULL;

	if( no >= 10 ){
		var1 = m_lpED->GetVariableNo();
		var2 = m_lpED->GetVariableNo();
	}else{
		p2 = m_lpED->GetExpValue();
		p3 = m_lpED->GetExpValue();
	}

	m_lpSD->GetUmapDriver()->Animation( no, p1, p2, p3, var1, var2 );
	TRACE( "command:VA %d( ... )\n", no );
}
/**********************************************************************
	jbg}bv`̈f[^Zbgi{bNXtBj
**********************************************************************/
void CCommandDriver::CommandVB( void )
{
	long page	= m_lpED->GetExpValue();
	long type	= m_lpED->GetExpValue();
	long x_pos	= m_lpED->GetExpValue();
	long y_pos	= m_lpED->GetExpValue();
	long x_size	= m_lpED->GetExpValue();
	long y_size = m_lpED->GetExpValue();
	long data	= m_lpED->GetExpValue();

	TRACE( "error:[Dummy] implement... /" );
	TRACE( "command:VB( ... )\n" );
}
/**********************************************************************
	jbg}bvfBXvC\̈̐ݒi]ݒj
**********************************************************************/
void CCommandDriver::CommandVC( void )
{
	long nPageNum	= m_lpED->GetExpValue();
	long x0Map		= m_lpED->GetExpValue();
	long y0Map		= m_lpED->GetExpValue();
	long cxMap		= m_lpED->GetExpValue();
	long cyMap		= m_lpED->GetExpValue();
	long cxUnit		= m_lpED->GetExpValue();
	long cyUnit		= m_lpED->GetExpValue();

	*m_lpED->GetSystemVar() = 
		m_lpSD->GetUmapDriver()->Create( nPageNum, x0Map, y0Map, cxMap, cyMap, cxUnit, cyUnit );
	TRACE( "command:VC( ... )\n" );
}
/**********************************************************************
	͈͎wtŃjbg}bv̓eʂɕ`悷
**********************************************************************/
void CCommandDriver::CommandVE( void )
{
	long pos_x	= m_lpED->GetExpValue(); // テjbg
	long pos_y	= m_lpED->GetExpValue();
	long len_x	= m_lpED->GetExpValue(); // jbg
	long len_y	= m_lpED->GetExpValue();
	long out_ptn= m_lpED->GetExpValue(); // ͈͊Ojbg
	long flag	= m_lpED->GetExpValue(); // PȂύX`

	m_lpSD->GetUmapDriver()->Draw( pos_x, pos_y, len_x, len_y, out_ptn, flag );
	TRACE( "command:VE( ... )\n" );
}
/**********************************************************************
	jbg}bv`
**********************************************************************/
void CCommandDriver::CommandVF( void )
{
	m_lpSD->GetUmapDriver()->Draw();
	TRACE( "command:VF( ... )\n" );
}
/**********************************************************************
	jbg}bv̒l̎擾
**********************************************************************/
void CCommandDriver::CommandVG( void )
{	
	long nPage	= m_lpED->GetExpValue();
	long nType	= m_lpED->GetExpValue();
	long x		= m_lpED->GetExpValue();
	long y		= m_lpED->GetExpValue();

	*m_lpED->GetSystemVar() = 
		m_lpSD->GetUmapDriver()->GetUnitmapData( nPage, nType, x, y);

	TRACE( "command:VG( ... )\n" );
}
/**********************************************************************
	yCg(VH
**********************************************************************/
void CCommandDriver::CommandVH( void )
{	
	long nPage	= m_lpED->GetExpValue();
	long x		= m_lpED->GetExpValue();
	long y		= m_lpED->GetExpValue();
	long lengs_x= m_lpED->GetExpValue();
	long lengs_y= m_lpED->GetExpValue();
	long max	= m_lpED->GetExpValue();

	m_lpSD->GetUmapDriver()->WalkPaint( nPage, x, y, lengs_x, lengs_y, max );
	TRACE( "command:VH( ... )\n" );
}
/**********************************************************************
	VF̃tOXVH
**********************************************************************/
void CCommandDriver::CommandVIC( void )
{
	long x		= m_lpED->GetExpValue();
	long y		= m_lpED->GetExpValue();
	long x_size	= m_lpED->GetExpValue();
	long y_size = m_lpED->GetExpValue();

	m_lpSD->GetUmapDriver()->Draw( x, y, x_size, y_size );
	TRACE( "command:VIC( ... )\n" );
}
void CCommandDriver::CommandVIP( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:VIP( ... )\n" );
}
/**********************************************************************
	dݕtyCg
**********************************************************************/
void CCommandDriver::CommandVJ( void )
{
	long page	= m_lpED->GetExpValue();
	long x		= m_lpED->GetExpValue();
	long y		= m_lpED->GetExpValue();
	long max	= m_lpED->GetExpValue();

	TRACE( "error:[Dummy] implement... /" );
	TRACE( "command:VJ( ... )\n" );
}
/**********************************************************************
	jbgbf擾ʒuяԐݒ聕XvCgFwi]ݒ)
**********************************************************************/
void CCommandDriver::CommandVP( void )
{
	long nPage  = m_lpED->GetExpValue();
	long x0Unit	= m_lpED->GetExpValue();
	long y0Unit = m_lpED->GetExpValue();
	long nxUnit = m_lpED->GetExpValue();
	long nyUnit = m_lpED->GetExpValue();
	long bSpCol = m_lpED->GetExpValue();

	m_lpSD->GetUmapDriver()->SetLayerState( nPage, x0Unit, y0Unit, nxUnit, nyUnit, bSpCol );
	TRACE( "command:VP( ... )\n" );
}
/**********************************************************************
	ϐ|MAPf[^]
**********************************************************************/
void CCommandDriver::CommandVR( void )
{
	long nPage = m_lpED->GetExpValue();
	long nType = m_lpED->GetExpValue();
	long *var  = m_lpED->GetVariableNo();

	m_lpSD->GetUmapDriver()->LoadMapData( nPage, nType, var );
	TRACE( "command:VR( ... )\n" );
}
/**********************************************************************
	jbg}bvւ̒l̃Zbg
**********************************************************************/
void CCommandDriver::CommandVS( void )
{
	long nPage	= m_lpED->GetExpValue();
	long nType	= m_lpED->GetExpValue();
	long x		= m_lpED->GetExpValue();
	long y		= m_lpED->GetExpValue();
	long wData	= m_lpED->GetExpValue();

	*m_lpED->GetSystemVar() = 
		m_lpSD->GetUmapDriver()->SetUnitmapData( nPage, nType, x, y, wData );
	
	TRACE( "command:VS( ... )\n" );
}
/**********************************************************************
	jbg}bv`̈f[^Rs[
**********************************************************************/
void CCommandDriver::CommandVT( void )
{
	long src_page = m_lpED->GetExpValue();
	long src_type = m_lpED->GetExpValue();
	long src_x	  = m_lpED->GetExpValue();
	long src_y	  = m_lpED->GetExpValue();
	long x_size	  = m_lpED->GetExpValue();
	long y_size	  = m_lpED->GetExpValue();
	long dst_page = m_lpED->GetExpValue();
	long dst_type = m_lpED->GetExpValue();
	long dst_x	  = m_lpED->GetExpValue();
	long dst_y	  = m_lpED->GetExpValue();

	m_lpSD->GetUmapDriver()->CopyUnitmapData( src_page, src_type, src_x, src_y,
											  x_size, y_size,
											  dst_page, dst_type, dst_x, dst_y );
	
	TRACE( "command:VT( ... )\n" );
}
/**********************************************************************
	jbg}bv̑wƂ̕\L̐؂ւ
**********************************************************************/
void CCommandDriver::CommandVV( void )
{
	long nPage = m_lpED->GetExpValue();
	long fEnable=m_lpED->GetExpValue();

	m_lpSD->GetUmapDriver()->SetLayerEnable( nPage, fEnable);
	TRACE( "command:VV( ... )\n" );
}
/**********************************************************************
	MAPϐf[^]
**********************************************************************/
void CCommandDriver::CommandVW( void )
{
	long nPage	= m_lpED->GetExpValue();
	long nType	= m_lpED->GetExpValue();
	long *var	= m_lpED->GetVariableNo();

	m_lpSD->GetUmapDriver()->StoreMapData( nPage, nType, var );
	TRACE( "command:VW( ... )\n" );
}

/**********************************************************************
	ύX
**********************************************************************/
void CCommandDriver::CommandVX( void )
{
//	long no = m_lpSD->GetParser()->GetC();
	long no = m_lpED->GetExpValue();
	
	switch( no ){
		case 0:
			{
				long nPage = m_lpED->GetExpValue();
				long x0Unit= m_lpED->GetExpValue();
				long y0Unit= m_lpED->GetExpValue();
				m_lpSD->GetUmapDriver()->SetLayer0Unit( nPage, x0Unit, y0Unit );
			}
			break;
		case 1:
			{
				long nPage = m_lpED->GetExpValue();
				long nxUnit= m_lpED->GetExpValue();
				long nyUnit= m_lpED->GetExpValue();
				m_lpSD->GetUmapDriver()->SetLayerUnitNum( nPage, nxUnit, nyUnit );
			}
			break;
		case 2:
			{
				long nPage = m_lpED->GetExpValue();
				long bSpCol= m_lpED->GetExpValue();
				long reserve=m_lpED->GetExpValue();
				m_lpSD->GetUmapDriver()->SetLayerSpriteColor( nPage, bSpCol );
			}
			break;
		case 3:
			{
				long nPage = m_lpED->GetExpValue();
				long reserve1 = m_lpED->GetExpValue();
				long reserve2 = m_lpED->GetExpValue();
			}
			break;
		default:
			{
				long p1 = m_lpED->GetExpValue();
				long p2 = m_lpED->GetExpValue();
				long p3 = m_lpED->GetExpValue();
			}
	}
	TRACE( "command:VX( ... )\n" );
}
void CCommandDriver::CommandVZ( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:VZ( ... )\n" );
}

