/*
 * UnitmapDriver.cpp - "ALICE SOFT System3.x" compatible scenario decoder SD35
 *  
 * cmdv.c  SYSTEM35 V command
 *
 * Copyright (C) 1997-1998 Masaki Chikama (Wren) <chikama@kasumi.ipl.mech.nagoya-u.ac.jp>
 *               1998-                           <masaki-c@is.aist-nara.ac.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *	99/06/19 Atsushi Mizuguchi EC++
 *
*/

#include "stdafx.h"
#include "UnitmapDriver.h"

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
namespace {
#endif
	const long ALSD_UNITMAP_ATTRIB_DEPTH = 4;
	const long WALKPAINT  = 2;
	const long WALKRESULT = 3;

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
};
#endif

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
using namespace ALSD;
using namespace ALSD_UNITMAP;
#endif
/**********************************************************************
	RXgNV
**********************************************************************/
CUnitmapDriver::CUnitmapDriver( CScenarioDecoder *_lpScenarioDecoder )
{
	m_lpSD			= _lpScenarioDecoder;
	memset( m_lpUnitMap, 0, sizeof(long*) * ALSD_UNITMAP_ATTRIB_DEPTH );
	m_lpLayerState = NULL;
	_vh_src		= NULL;
	_vh_dst		= NULL;
	memset( m_lpVAState, 0, sizeof(m_lpVAState) );
}

CUnitmapDriver::~CUnitmapDriver()
{
	Destroy();
}

/**********************************************************************
	j
**********************************************************************/
long CUnitmapDriver::Create( long _PageNum, long _x, long _y,
							 long _cw, long _ch, long _uw, long _uh )
{
	Destroy();
	
	m_PageNum	= _PageNum;
	m_x0Map		= _x;
	m_y0Map		= _y;
	m_cxMap		= _cw;
	m_cyMap		= _ch;
	m_cxUnit	= _uw;
	m_cyUnit	= _uh;
	
	// 
	long size = m_cxMap * m_cyMap * m_PageNum;
	for( long i = 0; i < ALSD_UNITMAP_ATTRIB_DEPTH; i++){
		m_lpUnitMap[i] = new long[size];
		memset( m_lpUnitMap[i], 0, sizeof(long) * size );
	}	
	m_lpLayerState = new stLayerState[m_PageNum];
	memset( m_lpLayerState, 0, sizeof(stLayerState) * m_PageNum );
	
	_vh_src = (stVHMark *)calloc(3 * (m_cxMap + m_cyMap), sizeof(stVHMark));
	_vh_dst = (stVHMark *)calloc(3 * (m_cxMap + m_cyMap), sizeof(stVHMark));
	//
	return 1;
}

void CUnitmapDriver::Destroy(void)
{
	for( long i = 0; i < ALSD_UNITMAP_ATTRIB_DEPTH; i++){
		if( m_lpUnitMap[i] )delete[] m_lpUnitMap[i];
	}
	if( m_lpLayerState ){
		delete[] m_lpLayerState;
	}

	memset( m_lpUnitMap, 0, sizeof(long*) * ALSD_UNITMAP_ATTRIB_DEPTH );
	m_lpLayerState = NULL;

	free( _vh_src );
	free( _vh_dst );
	_vh_src = NULL;
	_vh_dst = NULL;
}

/**********************************************************************
	C[Lݒ(VV
**********************************************************************/
void CUnitmapDriver::SetLayerEnable( long _PageNum, long _Enable )
{
	if( _Enable ){
		m_lpLayerState[_PageNum].bEnable = true;
	}else{
		m_lpLayerState[_PageNum].bEnable = false;
	}
}

/**********************************************************************
	C[ݒ(VP
**********************************************************************/
void CUnitmapDriver::SetLayerState ( long _PageNum, long _x0Unit, long _y0Unit, 
						 long _cxUnit,	long _cyUnit, long _SpColor )
{
	m_lpLayerState[_PageNum].x0Unit = _x0Unit;
	m_lpLayerState[_PageNum].y0Unit = _y0Unit;
	m_lpLayerState[_PageNum].nxUnit = _cxUnit;
	m_lpLayerState[_PageNum].nyUnit = _cyUnit;
	m_lpLayerState[_PageNum].SpColor= _SpColor;
	m_lpLayerState[_PageNum].useTTP = false;
	m_lpLayerState[_PageNum].bEnable= true;
}

/**********************************************************************
	ϐ}bvf[^
**********************************************************************/
void CUnitmapDriver::LoadMapData( long _PageNum, long _Type, long *_lpVar )
{
	long *dst = m_lpUnitMap[_Type-1] + (_PageNum * m_cxMap * m_cyMap );
	memcpy( dst, _lpVar, sizeof(long) * m_cxMap * m_cyMap );
}

/**********************************************************************
	}bvf[^ϐ
**********************************************************************/
void CUnitmapDriver::StoreMapData(long _PageNum, long _Type, long *_lpVar )
{
	long *src = m_lpUnitMap[_Type-1] + (_PageNum * m_cxMap * m_cyMap );
	memcpy( _lpVar, src, sizeof(long) * m_cxMap * m_cyMap );
}



/**********************************************************************
	VF
**********************************************************************/
void CUnitmapDriver::Draw( void )
{
	//
	long unit, unit_x, unit_y;
	stLayerState *lpLayer;

	for( long i = 0;i < m_PageNum; i++){
		lpLayer = &m_lpLayerState[i];
		if( lpLayer->bEnable == false ) continue;
		//
		for( long y = 0; y < m_cyMap; y++){
			for( long x = 0; x < m_cxMap; x++){
				//
				unit = *(m_lpUnitMap[0] + (i * m_cxMap * m_cyMap + (m_cxMap * y ) + x ));
				if( lpLayer->useTTP == true && lpLayer->TTPUnit == unit )continue;
				// jbg̎擾
				unit_x = unit % lpLayer->nxUnit;
				unit_y = unit / lpLayer->nxUnit;
				// XvCg]
				if( lpLayer->SpColor == 256 ){
					m_lpSD->GetGrpDriver()->BltFast(	lpLayer->x0Unit + m_cxUnit * unit_x,
														lpLayer->y0Unit + m_cyUnit * unit_y,
														m_cxUnit, m_cyUnit,
														m_x0Map + x * m_cxUnit,
														m_y0Map + y * m_cyUnit );
				
				}else{
					m_lpSD->GetGrpDriver()->BltSprite(	lpLayer->x0Unit + m_cxUnit * unit_x,
														lpLayer->y0Unit + m_cyUnit * unit_y,
														m_cxUnit, m_cyUnit,
														m_x0Map + x * m_cxUnit,
														m_y0Map + y * m_cyUnit,
														lpLayer->SpColor );
				}
			}
		}
	}	
	m_lpSD->GetGrpDriver()->OnDraw( m_x0Map, m_y0Map, m_cxMap * m_cxUnit, m_cyMap * m_cyUnit );
}

/**********************************************************************
	͈͎w`(VE
**********************************************************************/
void CUnitmapDriver::Draw( long _pos_x, long _pos_y,
						   long _len_x, long _len_y,
						   long _out_ptn, long _flag )
{
	
	//
	long unit, unit_x, unit_y;
	stLayerState *lpLayer;
	for( long i = 0;i < m_PageNum; i++){
		lpLayer = &m_lpLayerState[i];
		if( lpLayer->bEnable == false ) continue;
		//
		for( long y = _pos_y; y < (_pos_y + _len_y); y++){
			for( long x = _pos_x; x < (_pos_x + _len_x); x++){
				if( x >= m_cxMap || y >= m_cyMap ){
					unit = _out_ptn;		
				}else{
					unit = *(m_lpUnitMap[0] + (i * m_cxMap * m_cyMap + (m_cxMap * y ) + x ));
				}
				if( lpLayer->useTTP == true && lpLayer->TTPUnit == unit )continue;
				// jbg̎擾
				unit_x = unit % lpLayer->nxUnit;
				unit_y = unit / lpLayer->nxUnit;
				// XvCg]
				if( lpLayer->SpColor == 256 ){
					m_lpSD->GetGrpDriver()->BltFast(	lpLayer->x0Unit + m_cxUnit * unit_x,
														lpLayer->y0Unit + m_cyUnit * unit_y,
														m_cxUnit, m_cyUnit,
														m_x0Map + (x - _pos_x) * m_cxUnit,
														m_y0Map + (y - _pos_y) * m_cyUnit );
				
				}else{
					m_lpSD->GetGrpDriver()->BltSprite(	lpLayer->x0Unit + m_cxUnit * unit_x,
														lpLayer->y0Unit + m_cyUnit * unit_y,
														m_cxUnit, m_cyUnit,
														m_x0Map + ( x - _pos_x) * m_cxUnit,
														m_y0Map + ( y - _pos_y) * m_cyUnit,
														lpLayer->SpColor );
				}
			}
		}
	}
	m_lpSD->GetGrpDriver()->OnDraw( m_x0Map, m_y0Map, m_cxMap * m_cxUnit, m_cyMap * m_cyUnit );
}

/**********************************************************************
	ʍXVVIP
**********************************************************************/
void CUnitmapDriver::Draw( long _x, long _y, long _size_x, long _size_y )
{
	
	//
	long unit, unit_x, unit_y;
	stLayerState *lpLayer;
	for( long i = 0;i < m_PageNum; i++){
		lpLayer = &m_lpLayerState[i];
		if( lpLayer->bEnable == false ) continue;
		//
		for( long y = 0; y < _size_y; y++){
			for( long x = 0; x < _size_x; x++){
				unit = *(m_lpUnitMap[0] + (i * m_cxMap * m_cyMap + (m_cxMap * (_y + y) ) + _x + x ));

				if( lpLayer->useTTP == true && lpLayer->TTPUnit == unit )continue;
				// jbg̎擾
				unit_x = unit % lpLayer->nxUnit;
				unit_y = unit / lpLayer->nxUnit;
				// XvCg]
				if( lpLayer->SpColor == 256 ){
					m_lpSD->GetGrpDriver()->BltFast(	lpLayer->x0Unit + m_cxUnit * unit_x,
														lpLayer->y0Unit + m_cyUnit * unit_y,
														m_cxUnit, m_cyUnit,
														m_x0Map + (_x + x) * m_cxUnit,
														m_y0Map + (_y + y) * m_cyUnit );
				
				}else{
					m_lpSD->GetGrpDriver()->BltSprite(	lpLayer->x0Unit + m_cxUnit * unit_x,
														lpLayer->y0Unit + m_cyUnit * unit_y,
														m_cxUnit, m_cyUnit,
														m_x0Map + (_x - x) * m_cxUnit,
														m_y0Map + (_y - y) * m_cyUnit,
														lpLayer->SpColor );
				}
			}
		}
	}
	m_lpSD->GetGrpDriver()->OnDraw( m_x0Map, m_y0Map, m_cxMap * m_cxUnit, m_cyMap * m_cyUnit );
}



bool CUnitmapDriver::vh_checkImmovableArea(int page, int x, int y, int w, int h) {
	int _x, _y;
	
	if (x + w > m_cxMap) return true; 
	
	if (y + h > m_cyMap) return true; 

	for (_y = y; _y < (y+h); _y++) {
		for (_x = x; _x < (x+w); _x++) {
			if ( *(m_lpUnitMap[WALKRESULT] + (page * m_cxMap * m_cyMap) + ( m_cxMap * _y ) + _x ) == 255) return true;
		}
	}
	
	return false;
}


void CUnitmapDriver::vh_append_pos(int x, int y) {
	/* x, y  list ɒǉ */
	(vh_dst + vh_cnt_dst)->x = x;
	(vh_dst + vh_cnt_dst)->y = y;
	vh_cnt_dst++;
}

void CUnitmapDriver::vh_copy_to_src() {
	stVHMark *tmp;

	/* src list  dst list ̓ꊷ */
	tmp    = vh_src;
	vh_src = vh_dst;
	vh_dst = tmp;

	vh_cnt_src = vh_cnt_dst;
	vh_cnt_dst = 0;
}

#define UNITMAP_WALKRESULT(page,x,y) (m_lpUnitMap[WALKRESULT] + (page) * (m_cxMap * m_cyMap) + (y) * m_cxMap + (x))

void CUnitmapDriver::vh_check_udlr(int n, int nPage, int x, int y) {
	/* ㉺EnȂlZbglistɒǉ */
	if (x > 0) {
                if (*UNITMAP_WALKRESULT(nPage, x -1, y) == 0) {
                        *UNITMAP_WALKRESULT(nPage, x -1, y) = n;
                        vh_append_pos(x -1, y);
                }
        }
        if (x < (m_cxMap-1)) {
                if (*UNITMAP_WALKRESULT(nPage, x +1, y) == 0) {
                        *UNITMAP_WALKRESULT(nPage, x +1, y) = n;
                        vh_append_pos(x +1, y);
                }
        }
        if (y > 0) {
                if (*UNITMAP_WALKRESULT(nPage, x, y -1) == 0) {
                        *UNITMAP_WALKRESULT(nPage, x, y -1) = n;
                        vh_append_pos(x, y -1);
                }
        }
        if (y < (m_cyMap-1)) {
                if (*UNITMAP_WALKRESULT(nPage, x, y +1) == 0) {
                        *UNITMAP_WALKRESULT(nPage, x, y +1) = n;
                        vh_append_pos(x, y +1);
                }
	}
}

/**********************************************************************
	yCg
**********************************************************************/
void CUnitmapDriver::WalkPaint( long _PageNum, long _x, long _y,
							    long _lengs_x, long _lengs_y, long _max )
{
	long x, y;
	long lmtx, lmty, lmtw, lmth;
	if( _max > 255 )_max = 255;
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	lmtx = std::max(long(0), _x - _max );
	lmty = std::max(long(0), _y - _max );
	lmtw = std::min(_max + m_cxMap - _x, std::min(m_cxMap, long(2) * _max + long(1)));
	lmth = std::min(_max + m_cyMap - _y, std::min(m_cyMap, long(2) * _max + long(1)));
#else
	lmtx = max(0, _x - _max );
	lmty = max(0, _y - _max );
	lmtw = min(_max + m_cxMap - _x, min(m_cxMap, 2 * _max + 1));
	lmth = min(_max + m_cyMap - _y, min(m_cyMap, 2 * _max + 1));
#endif

	// ʂ̏
	for( y = 0; y < m_cyMap; y++){
		for( x = 0; x < m_cxMap; x++){
			if( _max < (abs(y - _y) + abs(x - _x))){
				*(m_lpUnitMap[WALKRESULT] + (_PageNum * m_cxMap * m_cyMap) + ( m_cxMap * y ) + x )
					= 255;
			}else if( *(m_lpUnitMap[WALKPAINT] + (_PageNum * m_cxMap * m_cyMap) + (m_cxMap * y) + x )
				>= 255 ){
				// ʍss
				*(m_lpUnitMap[WALKRESULT] + (_PageNum * m_cxMap * m_cyMap) + ( m_cxMap * y ) + x )
					= 255;
			}else{
				*(m_lpUnitMap[WALKRESULT] + (_PageNum * m_cxMap * m_cyMap) + ( m_cxMap * y ) + x )
					= 0;
			}
		}
	}
	// ړs\̈̌
	if( _lengs_y > 1 || _lengs_x > 1 ){
		for( y = 0; y < lmth; y++){
			for( x = 0; x < lmtw; x++){
				if( vh_checkImmovableArea( _PageNum, x+lmtx, y+lmty, _lengs_x, _lengs_y)){
					*(m_lpUnitMap[WALKRESULT] + (_PageNum * m_cxMap * m_cyMap) + ( m_cxMap * (y+lmty) ) + (x+lmtx) )
						= 255;
				}
			}
		}
	}
	/* list ̏ */
	long n;
	vh_src = _vh_src;
	vh_dst = _vh_dst;
	/* ʒu̐ݒ */
	vh_src -> x = _x;
	vh_src -> y = _y;
	vh_cnt_src = n = 1;
	vh_cnt_dst = 0;
	//
	while( 1 ){
		for( long i = 0; i < vh_cnt_src; i++){
			vh_check_udlr(n, _PageNum, (vh_src + i)->x, (vh_src + i)->y);
		}
		if (vh_cnt_dst == 0) break;
		vh_copy_to_src();
		if (n >= _max)    break;
		n++;
	}
	//
	/* ̈255 */
	for (y = 0; y < lmth; y++) {
		for (x = 0; x < lmtw; x++) {
			if (*UNITMAP_WALKRESULT(_PageNum, x + lmtx , y + lmty) == 0) {
				*UNITMAP_WALKRESULT(_PageNum, x + lmtx , y + lmty) = 255;
			}
		}
	}
	/* g 0 */
	*UNITMAP_WALKRESULT(_PageNum, _x , _y) = 0;
}

/**********************************************************************
	jbg}bv`̈f[^Rs[
**********************************************************************/
void CUnitmapDriver::CopyUnitmapData( long _src_page, long _src_type, long _src_x, long _src_y,
									  long _x_size, long _y_size,
									  long _dst_page, long _dst_type, long _dst_x, long _dst_y )
{
	long unit;
	for( long y = 0; y < _y_size; y++){
		for( long x = 0; x < _x_size; x++){
			unit = *(m_lpUnitMap[_src_type] + (_src_page * m_cxMap * m_cyMap) + (m_cxMap * (_src_y + y)) + x + _src_x);
			*(m_lpUnitMap[_dst_type] + (_dst_page * m_cxMap * m_cyMap) + (m_cxMap * (_dst_y + y)) + x + _dst_x ) = unit;
		}
	}
}

/**********************************************************************
	VAR}h
**********************************************************************/
void CUnitmapDriver::Animation( long _No,
							    long _param1, long _param2, long _param3,
								long *_var1, long *_var2)
{
	_param1--;
	switch( _No ){
		case 0:		// 
			Animation0( _param1, _param2, _param3 );
			break;

		case 1:		// \ʒu
			m_lpVAState[_param1].CurX
				= m_lpVAState[_param1].PreX
				= m_lpVAState[_param1].StartX = _param2;
			m_lpVAState[_param1].CurY
				= m_lpVAState[_param1].PreY
				= m_lpVAState[_param1].StartY = _param3;
			break;

		case 2:		// ړiI玩I~
			m_lpVAState[_param1].EndX = _param2;
			m_lpVAState[_param1].EndY = _param3;
			break;

		case 3:		// TCY
			m_lpVAState[_param1].Width = _param2;
			m_lpVAState[_param1].Height= _param3;
			break;

		case 4:		// p^[A`Ԋu(1/100)
			m_lpVAState[_param1].Pattern = _param2;
			m_lpVAState[_param1].Interval= _param3;
			break;

		case 5:		// 摜擾ʒu
			m_lpVAState[_param1].SrcX = _param2;
			m_lpVAState[_param1].SrcY = _param3;
			break;

		case 6:		// wiҔʒu
			m_lpVAState[_param1].SaveX = _param2;
			m_lpVAState[_param1].SaveY = _param3;
			break;

		case 7:		// XvCg@AF
			m_lpVAState[_param1].SpType = _param2;
			m_lpVAState[_param1].SpColor= _param3;
			break;

		case 10:	// Ԏ擾( _Var1: ~(0),is(1), _var2:ԍ)
			*_var1 = m_lpVAState[_param1].State;
			*_var2 = m_lpVAState[_param1].LapseFrame;
			break;

		case 11:	// ʒu擾
			*_var1 = m_lpVAState[_param1].CurX;
			*_var2 = m_lpVAState[_param1].CurY;
			break;

		default:
			break;
	}

}

/**********************************************************************
	VA 0
**********************************************************************/
void CUnitmapDriver::Animation0( long _param1, long _param2, long _param3 )
{
	if( _param2 == 0 ){
		// ~
		m_lpVAState[_param1].State = 0;
		m_lpVAState[_param1].Draw = true;
		if( _param3 == 0 ){
			// jbg
			m_lpSD->GetGrpDriver()->BltFast( m_lpVAState[_param1].SaveX, m_lpVAState[_param1].SaveY,
											 m_lpVAState[_param1].Width, m_lpVAState[_param1].Height,
											 m_lpVAState[_param1].PreX, m_lpVAState[_param1].PreY );
			m_lpSD->GetGrpDriver()->OnDraw(  m_lpVAState[_param1].PreX, m_lpVAState[_param1].PreY,
											 m_lpVAState[_param1].Width, m_lpVAState[_param1].Height);

		}else{
			// jbg`
			DrawVAUnit( _param1 );
			m_lpSD->GetGrpDriver()->OnDraw(  m_lpVAState[_param1].CurX, m_lpVAState[_param1].CurY,
											 m_lpVAState[_param1].Width, m_lpVAState[_param1].Height);
			
		}
		m_lpVAState[_param1].Draw = false;
	}else{
		stVAState &state = m_lpVAState[_param1];
		// `Jn
		state.LapseFrame = 0;
		state.LapseTime  = 0;
		state.TotalFrame = _param3;

		if( _param3 == 0 ){
			// PR}Ȃ
			state.NoMove = true;
		}else{
			if( state.StartX == state.EndX &&
				state.StartY == state.EndY ){
				//
				state.NoMove = true;
			}else{
				state.NoMove = false;
			}
		}
		// Aj[VJn
		//
		state.State = 1;
		state.Draw  = true;
		DrawVAUnit( _param1 );
		m_lpSD->GetGrpDriver()->OnDraw(  m_lpVAState[_param1].CurX, m_lpVAState[_param1].CurY,
										 m_lpVAState[_param1].Width, m_lpVAState[_param1].Height);
		
		//
		if( _param2 == 2 ){
			// L[
			m_lpSD->GetGrpDriver()->Sleep( 10 );
		}else if( _param2 == 3 ){
			// L[
			m_lpSD->GetGrpDriver()->Sleep( 10 );
		}
	}
}

/**********************************************************************
	
**********************************************************************/
void CUnitmapDriver::DrawVAUnit( long _no )
{
	long unitno = m_lpVAState[_no].LapseFrame % m_lpVAState[_no].Pattern;
	long unitX = unitno % 10;
	long unitY = unitno / 10;

	unitX = m_lpVAState[_no].SrcX + m_lpVAState[_no].Width * unitX;
	unitY = m_lpVAState[_no].SrcY + m_lpVAState[_no].Height * unitY;


	// Ҕ
	m_lpSD->GetGrpDriver()->BltFast( m_lpVAState[_no].CurX, m_lpVAState[_no].CurY,
									 m_lpVAState[_no].Width,m_lpVAState[_no].Height,
									 m_lpVAState[_no].SaveX,m_lpVAState[_no].SaveY );
	//
	if( m_lpVAState[_no].SpType == 0 ){
		m_lpSD->GetGrpDriver()->BltFast( unitX, unitY,
										 m_lpVAState[_no].Width,m_lpVAState[_no].Height,
										 m_lpVAState[_no].CurX,m_lpVAState[_no].CurY );
	}else if( m_lpVAState[_no].SpType == 1 ){
		m_lpSD->GetGrpDriver()->BltSprite( unitX, unitY,
										   m_lpVAState[_no].Width,m_lpVAState[_no].Height,
										   m_lpVAState[_no].CurX,m_lpVAState[_no].CurY,
										   m_lpVAState[_no].SpColor );
	}
}


