/*
 * UnitmapDriver.h - "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 */

#ifndef __UnitmapDriver_h__
#define __UnitmapDriver_h__
/**********************************************************************
	scenario decoder system unitmap driver
**********************************************************************/
#include "ScenarioDecoderSystem.h"

//
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
namespace ALSD {
#endif
	class CUnitmapDriver;
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
};
#endif

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
namespace ALSD_UNITMAP {
#endif
	struct stLayerState {
		bool bEnable;	// L/
		long x0Unit;	// jbgbfJnʒu
		long y0Unit;
		long nxUnit;	// jbgbfWJ
		long nyUnit;
		long SpColor;	// XvCgJ[pbgԍ
		bool useTTP;
		long TTPUnit;
	};

	struct stVHMark{
		long x;
		long y;
	};

	struct stVAState {
		long Width;		// 
		long Height;	// 
		long Pattern;	// p^[
		long Interval;	// `Ԋu
		long SrcX;		// 擾ʒu
		long SrcY;
		long StartX;	// \ʒu
		long StartY;
		long EndX;		// ړʒu
		long EndY;
		long SaveX;		// wiҔ
		long SaveY;
		long PreX;
		long PreY;
		long CurX;
		long CurY;
		long SpType;	// XvCgʒu
		long SpColor;	// XvCgF
		long State;		// O~@P
		long LapseFrame;// o߃R}
		long LapseTime;	// oߕb
		long TotalFrame;// t[
		bool Draw;		// `
		bool NoMove;	//
		bool UpdateView;//
	};


#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
};
#endif

//
// class
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
class ALSD::CUnitmapDriver {
#else
class CUnitmapDriver
#endif
public:
	// 
	CUnitmapDriver( CScenarioDecoder *_lpScenarioDecoder );
	virtual ~CUnitmapDriver();
	
	// j _cw, _ch(}bv̕(jbg) _uw, _uh (jbg̃TCY)
	long Create( long _PageNum, long _x, long _y, long _cw, long _ch, long _uw, long _uh );
	void Destroy( void );

	//
	void LoadMapData( long _PageNum, long _Type, long *_lpVar ); // Var->Map
	void StoreMapData(long _PageNum, long _Type, long *_lpVar ); // MAP->Var

	void Draw( void ); // VF
	void Draw( long _x, long _y, long _size_x, long _size_y );
	void Draw( long _pos_x, long _pos_y, long _len_x, long _len_y,
			   long _out_ptn, long _flag ); // VE
	//
	void WalkPaint( long _PageNum, long _x, long _y, long _lengs_x, long _lengs_y, long _max );
	void Animation( long _No, long _param1, long _param2, long _param3,
					long *_var1, long *_var2);
	
	
	//
	long GetUnitmapData( long _PageNum, long _Type, long _x, long _y ){
		return *(m_lpUnitMap[_Type-1] + (_PageNum * m_cxMap * m_cyMap) + (_y * m_cxMap) + _x );
	}
	long SetUnitmapData( long _PageNum, long _Type, long _x, long _y, long _data ){
		long ret = *(m_lpUnitMap[_Type-1] + (_PageNum * m_cxMap * m_cyMap) + (_y * m_cxMap) + _x );
		*(m_lpUnitMap[_Type-1] + (_PageNum * m_cxMap * m_cyMap) + (_y * m_cxMap) + _x ) = _data;
		return ret;
	}
	void CopyUnitmapData( long _src_page, long _src_type, long _src_x, long _src_y, long _x_size, long _y_size,
						  long _dst_page, long _dst_type, long _dst_x, long _dst_y );


	// C[ݒ
	void SetLayerEnable( long _PageNum, long _Enable );
	void SetLayerState ( long _PageNum, long _x0Unit, long _y0Unit, 
						 long _cxUnit,	long _cyUnit, long _SpColor );
	void SetLayer0Unit(  long _PageNum, long _x0Unit, long _y0Unit){
		m_lpLayerState[_PageNum].x0Unit = _x0Unit;
		m_lpLayerState[_PageNum].y0Unit = _y0Unit;
	}
	void SetLayerUnitNum(long _PageNum, long _nxUnit, long _nyUnit ){
		m_lpLayerState[_PageNum].nxUnit = _nxUnit;
		m_lpLayerState[_PageNum].nyUnit = _nyUnit;
	}
	void SetLayerSpriteColor( long _PageNum, long _SpCol ){
		m_lpLayerState[_PageNum].SpColor = _SpCol;
	}
	CScenarioDecoder	*m_lpSD;

	//
	long m_PageNum;	// C[
	long m_x0Map;	// _
	long m_y0Map;
	long m_cxMap;	// }bv̕ijbgPʁj
	long m_cyMap;
	long m_cxUnit;	// jbgTCY
	long m_cyUnit;
	
	long *m_lpUnitMap[4];	// jbg}bv
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	ALSD_UNITMAP::stLayerState *m_lpLayerState;	// C[
	ALSD_UNITMAP::stVHMark	*vh_src, *vh_dst, *_vh_src, *_vh_dst;
	ALSD_UNITMAP::stVAState	m_lpVAState[32];
#else
	stLayerState *m_lpLayerState;	// C[
	stVHMark	*vh_src, *vh_dst, *_vh_src, *_vh_dst;
	stVAState	m_lpVAState[32];
#endif
	long		vh_cnt_src, vh_cnt_dst;
	//
	bool vh_checkImmovableArea(int page, int x, int y, int w, int h);
	void vh_append_pos(int x, int y);
	void vh_copy_to_src();
	void vh_check_udlr(int n, int nPage, int x, int y);
	
	void Animation0( long _param1, long _param2, long _param3 );
	void DrawVAUnit( long _no );
private:
	//

};

#endif
